"""Basic layout components.
"""
import grok
from hurry import yui
from hurry.resource import mode
from megrok.layout import Page, Layout, EditForm, DisplayForm, AddForm
from z3c.flashmessage.interfaces import IMessageSource, IMessageReceiver
from zope.component import getUtility, queryUtility, ComponentLookupError
from zope.interface import Interface
try:
    from zope.site.hooks import getSite
except ImportError:
    # BBB
    from zope.app.component.hooks import getSite
from zope.formlib.utility import setUpWidgets    
from waeup.sirp.interfaces import IWAeUPObject
from waeup.sirp.browser.interfaces import ITheme
from waeup.sirp.browser.theming import get_all_themes, WAeUPThemeGray1

grok.templatedir('templates')

def NullValidator(*args, **kw):
    """A validator that does not validate.

    This is needed especially for cancel buttons. We don't want data
    to be validated that will be thrown away in the next step.

    You can use it with ``grok.action`` decorator like this::

      @grok.action('Cancel', validator=NullValidator)
      def cancel(self, **data):
        self.redirect(<whereever-you-go>)
    """
    return dict()

class Messages(grok.View):
    """Display messages of message receivers.
    """

    grok.context(Interface)

    @property
    def messages(self):
        receiver = getUtility(IMessageReceiver)
        return receiver.receive()

class UtilityView(object):
    """A view mixin with useful methods.

    The ``pnav`` attribute (a number) tells, to which primary
    navigation tab a page declares to belong.
    """
    title = u'' # What appears in the content title...
    pnav = 0 # Primary navigation index...
    
    def application_url(self, name=None):
        """Return the URL of the nearest site.
        """
        site = getSite()
        #if not site:
        #    raise ComponentLookupError("No site found.")
        return self.url(site, name)

    def flash(self, message, type='message'):
        """Send a short message to the user.
        """
        source = queryUtility(IMessageSource, name='session')
        if source is None:
            return None
        source.send(message, type)
        return True

class WAeUPLayout(Layout, UtilityView):
    """A megrok.layout.Layout with additional methods.
    """
    grok.baseclass()
    
class WAeUPPage(Page):
    """A megrok.layout page with additional methods.
    """
    grok.baseclass()

class WAeUPDisplayFormPage(DisplayForm, UtilityView):
    """A megrok.layout.DisplayForm with additional methods.
    """
    grok.baseclass()
    grok.template('default_waeup_display_form')

class WAeUPEditFormPage(EditForm, UtilityView):
    """A megrok.layout.EditForm with additional methods.
    """
    grok.baseclass()
    grok.template('default_waeup_edit_form')
    
    def setUpWidgets(self,ignore_request=False):
        super(WAeUPEditFormPage,self).setUpWidgets(ignore_request)
        if self.widgets.get('title'):
            self.widgets['title'].displayWidth = 80

class WAeUPAddFormPage(AddForm,  UtilityView):
    """A megrok.layout.AddForm with additional methods.
    """
    grok.baseclass()
    grok.template('default_waeup_edit_form')

class SiteLayout(WAeUPLayout):
    """ The general site layout.
    """
    grok.context(IWAeUPObject)

    #: An instance of the default theme to use for the site layout
    default_theme = WAeUPThemeGray1()

    @property
    def site(self):
        return grok.getSite()
    
    def getAppTitle(self):
        return getattr(grok.getSite(), 'name', u'Sample University')
        
    def isAuthenticated(self):
        """Return True if the calling user is authenticated.
        """
        usertitle = self.request.principal.title
        return usertitle != 'Unauthenticated User'
    
    def getUserTitle(self):
        """Return principal title of current user.
        """
        usertitle = self.request.principal.title
        if usertitle == 'Unauthenticated User':
            return u'Anonymous User'
        return usertitle

    def getUserId(self):
        """Return id of current user.
        """
        userid = self.request.principal.id
        return userid

    def update(self):
        """Include the resources required by the chosen skin/theme.

        University instances provide a 'skin' attribute, that should
        hold the internal name of a theme.

        A theme in the waeup.sirp sense consists of a list of
        CSS/JavaScript resources defined in the
        :mod:`waeup.sirp.browser.resources` module.

        If the context University object has no such attribute or the
        set value is not a valid theme name, we pick 'gray waeup
        theme' as default.
        """
        mode('minified')
        theme_name = getattr(grok.getSite(), 'skin', '')
        theme = queryUtility(ITheme, name=theme_name,
                             default=self.default_theme)
        for resource in theme.getResources():
            resource.need()
        return
